class COM_CONNECTION:
    def __init__(self, data: dict):
        self.NAME = data["NAME"]
        self.METHOD = "COM"
        self.PORT = data["PORT"]
        self.BAUDRATE = data["BAUDRATE"]
        self.DEVICE = data["DEVICE"]+"_serial"
        self.EXECPASS = data["EXECPASS"] if data.get("EXECPASS") else ""

    def __str__(self):
        return f"{self.METHOD}, {self.PORT}, {self.BAUDRATE}, {self.DEVICE}, {self.EXECPASS}"

    def getNetmikoConnDict(self):
        return {
            'device_type': self.DEVICE,
            'serial_settings': {
                'port': self.PORT,
                'baudrate': self.BAUDRATE,
            },
            'username': '',
            'password': '',
            'secret': self.EXECPASS,
        }

class SSH_CONNECTION:
    def __init__(self, data: dict):
        self.NAME = data["NAME"]
        self.METHOD = data["METHOD"]
        self.HOST = data["HOST"]
        self.PORT = data["PORT"]
        self.USERNAME = data["USERNAME"]
        self.PASSWORD = data["PASSWORD"]
        self.DEVICE = data["DEVICE"]
        self.EXECPASS = data["EXECPASS"] if data.get("EXECPASS") else ""

    def __str__(self):
        return f"{self.METHOD}, {self.HOST}, {self.PORT}, {self.USERNAME}, {self.PASSWORD}, {self.EXECPASS}"

    def getNetmikoConnDict(self):
        return {
            'device_type': self.DEVICE,
            'host': self.HOST,
            'port': self.PORT,
            'username': self.USERNAME,
            'password': self.PASSWORD,
            'secret': self.EXECPASS,
        }

class TELNET_CONNECTION:
    def __init__(self, data: dict):
        self.NAME = data["NAME"]
        self.METHOD = data["METHOD"]
        self.HOST = data["HOST"]
        self.PORT = data["PORT"]
        self.PASSWORD = data["PASSWORD"]
        self.DEVICE = data["DEVICE"] + "_telnet"
        self.EXECPASS = data["EXECPASS"] if data.get("EXECPASS") else ""

    def __str__(self):
        return f"{self.METHOD}, {self.HOST}, {self.PORT}, {self.PASSWORD}, {self.EXECPASS}"

    def getNetmikoConnDict(self):
        return {
            'device_type': self.DEVICE,
            'host': self.HOST,
            'port': self.PORT,
            'password': self.PASSWORD,
            'secret': self.EXECPASS,
        }

class TFTP_CONNECTION:
    pass


class INPUT:
    type = None
    id = None
    label = None

    def read(self, line: str):
        line = line.strip().split(";")
        self.type, self.id, self.label = line[0], line[1], line[2]

    def toTuple(self):
        return self.type, self.id, self.label

    def __str__(self):
        return f"{self.type};{self.id};{self.label}"

class COMMAND:
    def __init__(self, meta_section: dict, commands_section: list[str], devices_section: list[str], inputs_section: list[INPUT]):
        self.NAME = meta_section["NAME"]
        self.TYPE = meta_section["TYPE"]
        self.commands = commands_section
        self.devices = devices_section
        self.inputs_section = inputs_section

    def __str__(self):
        return f"{self.NAME} {self.TYPE} {self.commands} {self.devices} {self.inputs_section}"




from NetManage.utils.commands import COMMAND, INPUT


def read_NMCOMM(file:str) -> COMMAND:
    meta_section = {}
    commands_section = []
    devices_section = []
    inputs_section = []

    with open(file, 'r') as f:
        lines = f.readlines()
        curr_section = None
        for idx, line in enumerate(lines):
            line = line.strip()
            print(line)
            if line == "-- META":
                print("META")
                curr_section = meta_section
            elif line == "-- COMMANDS":
                print("COMMAND")
                curr_section = commands_section
            elif line == "-- DEVICES":
                print("DEVICES")
                curr_section = devices_section
            elif line == "-- INPUTS":
                print("INPUTS")
                curr_section = inputs_section

            if line[0:2] == '--':
                continue

            if curr_section is meta_section:
                if len(line) > 0:
                    key, value = line.split(': ')
                    print(key, value)
                    curr_section[key] = value

            if curr_section is commands_section or curr_section is devices_section:
                curr_section.append(line)

            if curr_section is inputs_section:
                new_input = INPUT()
                new_input.read(line)
                curr_section.append(new_input)


    if not validate_NMCOMM_file(meta_section, commands_section, devices_section, inputs_section):
        raise Exception("Invalid NMCOM file")

    command = COMMAND(meta_section, commands_section, devices_section, inputs_section)
    print(inputs_section[0])
    return command

def validate_NMCOMM_file(meta_section, commands_section, devices_section, inputs_section):
    meta_keys = meta_section.keys()

    if not (len(meta_keys) == 2 and "NAME" in meta_keys and "TYPE" in meta_keys):
        return False
    if len(commands_section) == 0:
        return False
    if len(devices_section) == 0:
        return False
    return True

def write_NMCOMM(output:str, command_obj: COMMAND):
    with open(output, 'w+', encoding="UTF-8") as f:
        f.writelines([
            "-- META\n",
            f"NAME: {command_obj.NAME}\n",
            f"TYPE: {command_obj.TYPE}\n",
        ])
        f.write("-- COMMANDS\n")
        f.writelines([f"{cmd}\n" for cmd in command_obj.commands])
        f.write("-- DEVICES\n")
        f.writelines([f"{device}\n" for device in command_obj.devices])
        f.write("-- INPUTS\n")
        f.writelines([f"{inpt}\n" for inpt in command_obj.inputs_section])

if __name__ == '__main__':

    input1 = INPUT()
    input1.read("TEXT;hostname;Nazwa hosta:")
    input2 = INPUT()
    input2.read("SUBNET;subnet;Maska:")

    command = COMMAND(
        meta_section={
            "TYPE": "CONFIG",
            "NAME": "TEST"
        },
        commands_section=[
            "hostname 1",
            "exit 2"
        ],
        devices_section=[
            "cisco_ios",
            "cisco_xe",
            "cisco_asa"
        ],
        inputs_section=[input1, input2]
    )
    #write_NMCOMM("../commands/test2.nmcomm", command)
    read_NMCOMM("../commands/komenda1.nmcomm")